@@ -15,7 +15,7 @@ Predictor <- R6::R6Class(
 
         # Freeing up handle
         lgb.call(
-          "LGBM_BoosterFree_R"
+          fun_name = "LGBM_BoosterFree_R"
           , ret = NULL
           , private$handle
         )
@@ -30,14 +30,14 @@ Predictor <- R6::R6Class(
       params <- list(...)
       private$params <- lgb.params2str(params)
       # Create new lgb handle
-      handle <- 0.0
+      handle <- lgb.null.handle()
 
       # Check if handle is a character
       if (is.character(modelfile)) {
 
         # Create handle on it
         handle <- lgb.call(
-          "LGBM_BoosterCreateFromModelfile_R"
+          fun_name = "LGBM_BoosterCreateFromModelfile_R"
           , ret = handle
           , lgb.c_str(modelfile)
         )
@@ -66,7 +66,7 @@ Predictor <- R6::R6Class(
 
       cur_iter <- 0L
       lgb.call(
-        "LGBM_BoosterGetCurrentIteration_R"
+        fun_name = "LGBM_BoosterGetCurrentIteration_R"
         , ret = cur_iter
         , private$handle
       )
@@ -103,7 +103,7 @@ Predictor <- R6::R6Class(
 
         # Predict from temporary file
         lgb.call(
-          "LGBM_BoosterPredictForFile_R"
+          fun_name = "LGBM_BoosterPredictForFile_R"
           , ret = NULL
           , private$handle
           , data
@@ -131,7 +131,7 @@ Predictor <- R6::R6Class(
 
         # Check number of predictions to do
         npred <- lgb.call(
-          "LGBM_BoosterCalcNumPredict_R"
+          fun_name = "LGBM_BoosterCalcNumPredict_R"
           , ret = npred
           , private$handle
           , as.integer(num_row)
@@ -153,7 +153,7 @@ Predictor <- R6::R6Class(
             storage.mode(data) <- "double"
           }
           preds <- lgb.call(
-            "LGBM_BoosterPredictForMat_R"
+            fun_name = "LGBM_BoosterPredictForMat_R"
             , ret = preds
             , private$handle
             , data
@@ -173,7 +173,7 @@ Predictor <- R6::R6Class(
           }
           # Check if data is a dgCMatrix (sparse matrix, column compressed format)
           preds <- lgb.call(
-            "LGBM_BoosterPredictForCSC_R"
+            fun_name = "LGBM_BoosterPredictForCSC_R"
             , ret = preds
             , private$handle
             , data@p
