@@ -111,6 +111,22 @@ lgb.cv <- function(params = list()
   fobj <- NULL
   eval_functions <- list(NULL)
 
+  # set some parameters, resolving the way they were passed in with other parameters
+  # in `params`.
+  # this ensures that the model stored with Booster$save() correctly represents
+  # what was passed in
+  params <- lgb.check.wrapper_param(
+    main_param_name = "num_iterations"
+    , params = params
+    , alternative_kwarg_value = nrounds
+  )
+  params <- lgb.check.wrapper_param(
+    main_param_name = "early_stopping_round"
+    , params = params
+    , alternative_kwarg_value = early_stopping_rounds
+  )
+  early_stopping_rounds <- params[["early_stopping_round"]]
+
   # Check for objective (function or not)
   if (is.function(params$objective)) {
     fobj <- params$objective
@@ -146,13 +162,7 @@ lgb.cv <- function(params = list()
   if (!is.null(predictor)) {
     begin_iteration <- predictor$current_iter() + 1L
   }
-  # Check for number of rounds passed as parameter - in case there are multiple ones, take only the first one
-  n_trees <- .PARAMETER_ALIASES()[["num_iterations"]]
-  if (any(names(params) %in% n_trees)) {
-    end_iteration <- begin_iteration + params[[which(names(params) %in% n_trees)[1L]]] - 1L
-  } else {
-    end_iteration <- begin_iteration + nrounds - 1L
-  }
+  end_iteration <- begin_iteration + params[["num_iterations"]] - 1L
 
   # Check interaction constraints
   cnames <- NULL
@@ -227,18 +237,8 @@ lgb.cv <- function(params = list()
     callbacks <- add.cb(callbacks, cb.record.evaluation())
   }
 
-  # If early stopping was passed as a parameter in params(), prefer that to keyword argument
-  # early_stopping_rounds by overwriting the value in 'early_stopping_rounds'
-  early_stop <- .PARAMETER_ALIASES()[["early_stopping_round"]]
-  early_stop_param_indx <- names(params) %in% early_stop
-  if (any(early_stop_param_indx)) {
-    first_early_stop_param <- which(early_stop_param_indx)[[1L]]
-    first_early_stop_param_name <- names(params)[[first_early_stop_param]]
-    early_stopping_rounds <- params[[first_early_stop_param_name]]
-  }
-
   # Did user pass parameters that indicate they want to use early stopping?
-  using_early_stopping_via_args <- !is.null(early_stopping_rounds) && early_stopping_rounds > 0L
+  using_early_stopping <- !is.null(early_stopping_rounds) && early_stopping_rounds > 0L
 
   boosting_param_names <- .PARAMETER_ALIASES()[["boosting"]]
   using_dart <- any(
@@ -253,7 +253,7 @@ lgb.cv <- function(params = list()
   # Cannot use early stopping with 'dart' boosting
   if (using_dart) {
     warning("Early stopping is not available in 'dart' mode.")
-    using_early_stopping_via_args <- FALSE
+    using_early_stopping <- FALSE
 
     # Remove the cb.early.stop() function if it was passed in to callbacks
     callbacks <- Filter(
@@ -265,7 +265,7 @@ lgb.cv <- function(params = list()
   }
 
   # If user supplied early_stopping_rounds, add the early stopping callback
-  if (using_early_stopping_via_args) {
+  if (using_early_stopping) {
     callbacks <- add.cb(
       callbacks
       , cb.early.stop(
