@@ -6,17 +6,16 @@
 
 #include <LightGBM/network.h>
 #include <LightGBM/utils/array_args.h>
+#include <LightGBM/utils/json11.h>
 #include <LightGBM/utils/log.h>
 #include <LightGBM/utils/openmp_wrapper.h>
 
 #include <fstream>
 
-#include <LightGBM/json11.hpp>
-
-using namespace json11;
-
 namespace LightGBM {
 
+using json11::Json;
+
 DatasetLoader::DatasetLoader(const Config& io_config, const PredictFunction& predict_fun, int num_class, const char* filename)
   :config_(io_config), random_(config_.data_random_seed), predict_fun_(predict_fun), num_class_(num_class) {
   label_idx_ = 0;
@@ -165,7 +164,17 @@ void DatasetLoader::SetHeader(const char* filename) {
   }
 }
 
-Dataset* DatasetLoader::LoadFromFile(const char* filename, const char* initscore_file, int rank, int num_machines) {
+void CheckSampleSize(size_t sample_cnt, size_t num_data) {
+  if (static_cast<double>(sample_cnt) / num_data < 0.2f &&
+      sample_cnt < 100000) {
+    Log::Warning(
+        "Using too small ``bin_construct_sample_cnt`` may encounter "
+        "unexpected "
+        "errors and poor accuracy.");
+  }
+}
+
+Dataset* DatasetLoader::LoadFromFile(const char* filename, int rank, int num_machines) {
   // don't support query id in data file when training in parallel
   if (num_machines > 1 && !config_.pre_partition) {
     if (group_idx_ > 0) {
@@ -184,13 +193,15 @@ Dataset* DatasetLoader::LoadFromFile(const char* filename, const char* initscore
     }
     dataset->data_filename_ = filename;
     dataset->label_idx_ = label_idx_;
-    dataset->metadata_.Init(filename, initscore_file);
+    dataset->metadata_.Init(filename);
     if (!config_.two_round) {
       // read data to memory
       auto text_data = LoadTextDataToMemory(filename, dataset->metadata_, rank, num_machines, &num_global_data, &used_data_indices);
       dataset->num_data_ = static_cast<data_size_t>(text_data.size());
       // sample data
       auto sample_data = SampleTextDataFromMemory(text_data);
+      CheckSampleSize(sample_data.size(),
+                      static_cast<size_t>(dataset->num_data_));
       // construct feature bin mappers
       ConstructBinMappersFromTextData(rank, num_machines, sample_data, parser.get(), dataset.get());
       // initialize label
@@ -206,10 +217,13 @@ Dataset* DatasetLoader::LoadFromFile(const char* filename, const char* initscore
       } else {
         dataset->num_data_ = num_global_data;
       }
+      CheckSampleSize(sample_data.size(),
+                      static_cast<size_t>(dataset->num_data_));
       // construct feature bin mappers
       ConstructBinMappersFromTextData(rank, num_machines, sample_data, parser.get(), dataset.get());
       // initialize label
       dataset->metadata_.Init(dataset->num_data_, weight_idx_, group_idx_);
+      Log::Debug("Making second pass...");
       // extract features
       ExtractFeaturesFromFile(filename, parser.get(), used_data_indices, dataset.get());
     }
@@ -226,7 +240,7 @@ Dataset* DatasetLoader::LoadFromFile(const char* filename, const char* initscore
 
 
 
-Dataset* DatasetLoader::LoadFromFileAlignWithOtherDataset(const char* filename, const char* initscore_file, const Dataset* train_data) {
+Dataset* DatasetLoader::LoadFromFileAlignWithOtherDataset(const char* filename, const Dataset* train_data) {
   data_size_t num_global_data = 0;
   std::vector<data_size_t> used_data_indices;
   auto dataset = std::unique_ptr<Dataset>(new Dataset());
@@ -238,7 +252,7 @@ Dataset* DatasetLoader::LoadFromFileAlignWithOtherDataset(const char* filename,
     }
     dataset->data_filename_ = filename;
     dataset->label_idx_ = label_idx_;
-    dataset->metadata_.Init(filename, initscore_file);
+    dataset->metadata_.Init(filename);
     if (!config_.two_round) {
       // read data in memory
       auto text_data = LoadTextDataToMemory(filename, dataset->metadata_, 0, 1, &num_global_data, &used_data_indices);
@@ -286,8 +300,10 @@ Dataset* DatasetLoader::LoadFromBinFile(const char* data_filename, const char* b
 
   // check token
   size_t size_of_token = std::strlen(Dataset::binary_file_token);
-  size_t read_cnt = reader->Read(buffer.data(), sizeof(char) * size_of_token);
-  if (read_cnt != sizeof(char) * size_of_token) {
+  size_t read_cnt = reader->Read(
+      buffer.data(),
+      VirtualFileWriter::AlignedSize(sizeof(char) * size_of_token));
+  if (read_cnt < sizeof(char) * size_of_token) {
     Log::Fatal("Binary file error: token has the wrong size");
   }
   if (std::string(buffer.data()) != std::string(Dataset::binary_file_token)) {
@@ -317,55 +333,59 @@ Dataset* DatasetLoader::LoadFromBinFile(const char* data_filename, const char* b
   // get header
   const char* mem_ptr = buffer.data();
   dataset->num_data_ = *(reinterpret_cast<const data_size_t*>(mem_ptr));
-  mem_ptr += sizeof(dataset->num_data_);
+  mem_ptr += VirtualFileWriter::AlignedSize(sizeof(dataset->num_data_));
   dataset->num_features_ = *(reinterpret_cast<const int*>(mem_ptr));
-  mem_ptr += sizeof(dataset->num_features_);
+  mem_ptr += VirtualFileWriter::AlignedSize(sizeof(dataset->num_features_));
   dataset->num_total_features_ = *(reinterpret_cast<const int*>(mem_ptr));
-  mem_ptr += sizeof(dataset->num_total_features_);
+  mem_ptr +=
+      VirtualFileWriter::AlignedSize(sizeof(dataset->num_total_features_));
   dataset->label_idx_ = *(reinterpret_cast<const int*>(mem_ptr));
-  mem_ptr += sizeof(dataset->label_idx_);
+  mem_ptr += VirtualFileWriter::AlignedSize(sizeof(dataset->label_idx_));
   dataset->max_bin_ = *(reinterpret_cast<const int*>(mem_ptr));
-  mem_ptr += sizeof(dataset->max_bin_);
+  mem_ptr += VirtualFileWriter::AlignedSize(sizeof(dataset->max_bin_));
   dataset->bin_construct_sample_cnt_ = *(reinterpret_cast<const int*>(mem_ptr));
-  mem_ptr += sizeof(dataset->bin_construct_sample_cnt_);
+  mem_ptr += VirtualFileWriter::AlignedSize(
+      sizeof(dataset->bin_construct_sample_cnt_));
   dataset->min_data_in_bin_ = *(reinterpret_cast<const int*>(mem_ptr));
-  mem_ptr += sizeof(dataset->min_data_in_bin_);
+  mem_ptr += VirtualFileWriter::AlignedSize(sizeof(dataset->min_data_in_bin_));
   dataset->use_missing_ = *(reinterpret_cast<const bool*>(mem_ptr));
-  mem_ptr += sizeof(dataset->use_missing_);
+  mem_ptr += VirtualFileWriter::AlignedSize(sizeof(dataset->use_missing_));
   dataset->zero_as_missing_ = *(reinterpret_cast<const bool*>(mem_ptr));
-  mem_ptr += sizeof(dataset->zero_as_missing_);
-  dataset->sparse_threshold_ = *(reinterpret_cast<const double*>(mem_ptr));
-  mem_ptr += sizeof(dataset->sparse_threshold_);
+  mem_ptr += VirtualFileWriter::AlignedSize(sizeof(dataset->zero_as_missing_));
   const int* tmp_feature_map = reinterpret_cast<const int*>(mem_ptr);
   dataset->used_feature_map_.clear();
   for (int i = 0; i < dataset->num_total_features_; ++i) {
     dataset->used_feature_map_.push_back(tmp_feature_map[i]);
   }
-  mem_ptr += sizeof(int) * dataset->num_total_features_;
+  mem_ptr += VirtualFileWriter::AlignedSize(sizeof(int) *
+                                            dataset->num_total_features_);
   // num_groups
   dataset->num_groups_ = *(reinterpret_cast<const int*>(mem_ptr));
-  mem_ptr += sizeof(dataset->num_groups_);
+  mem_ptr += VirtualFileWriter::AlignedSize(sizeof(dataset->num_groups_));
   // real_feature_idx_
   const int* tmp_ptr_real_feature_idx_ = reinterpret_cast<const int*>(mem_ptr);
   dataset->real_feature_idx_.clear();
   for (int i = 0; i < dataset->num_features_; ++i) {
     dataset->real_feature_idx_.push_back(tmp_ptr_real_feature_idx_[i]);
   }
-  mem_ptr += sizeof(int) * dataset->num_features_;
+  mem_ptr +=
+      VirtualFileWriter::AlignedSize(sizeof(int) * dataset->num_features_);
   // feature2group
   const int* tmp_ptr_feature2group = reinterpret_cast<const int*>(mem_ptr);
   dataset->feature2group_.clear();
   for (int i = 0; i < dataset->num_features_; ++i) {
     dataset->feature2group_.push_back(tmp_ptr_feature2group[i]);
   }
-  mem_ptr += sizeof(int) * dataset->num_features_;
+  mem_ptr +=
+      VirtualFileWriter::AlignedSize(sizeof(int) * dataset->num_features_);
   // feature2subfeature
   const int* tmp_ptr_feature2subfeature = reinterpret_cast<const int*>(mem_ptr);
   dataset->feature2subfeature_.clear();
   for (int i = 0; i < dataset->num_features_; ++i) {
     dataset->feature2subfeature_.push_back(tmp_ptr_feature2subfeature[i]);
   }
-  mem_ptr += sizeof(int) * dataset->num_features_;
+  mem_ptr +=
+      VirtualFileWriter::AlignedSize(sizeof(int) * dataset->num_features_);
   // group_bin_boundaries
   const uint64_t* tmp_ptr_group_bin_boundaries = reinterpret_cast<const uint64_t*>(mem_ptr);
   dataset->group_bin_boundaries_.clear();
@@ -380,7 +400,8 @@ Dataset* DatasetLoader::LoadFromBinFile(const char* data_filename, const char* b
   for (int i = 0; i < dataset->num_groups_; ++i) {
     dataset->group_feature_start_.push_back(tmp_ptr_group_feature_start[i]);
   }
-  mem_ptr += sizeof(int) * (dataset->num_groups_);
+  mem_ptr +=
+      VirtualFileWriter::AlignedSize(sizeof(int) * (dataset->num_groups_));
 
   // group_feature_cnt_
   const int* tmp_ptr_group_feature_cnt = reinterpret_cast<const int*>(mem_ptr);
@@ -388,55 +409,12 @@ Dataset* DatasetLoader::LoadFromBinFile(const char* data_filename, const char* b
   for (int i = 0; i < dataset->num_groups_; ++i) {
     dataset->group_feature_cnt_.push_back(tmp_ptr_group_feature_cnt[i]);
   }
-  mem_ptr += sizeof(int) * (dataset->num_groups_);
-
-  if (!config_.monotone_constraints.empty()) {
-    CHECK(static_cast<size_t>(dataset->num_total_features_) == config_.monotone_constraints.size());
-    dataset->monotone_types_.resize(dataset->num_features_);
-    for (int i = 0; i < dataset->num_total_features_; ++i) {
-      int inner_fidx = dataset->InnerFeatureIndex(i);
-      if (inner_fidx >= 0) {
-        dataset->monotone_types_[inner_fidx] = config_.monotone_constraints[i];
-      }
-    }
-  } else {
-    const int8_t* tmp_ptr_monotone_type = reinterpret_cast<const int8_t*>(mem_ptr);
-    dataset->monotone_types_.clear();
-    for (int i = 0; i < dataset->num_features_; ++i) {
-      dataset->monotone_types_.push_back(tmp_ptr_monotone_type[i]);
-    }
-  }
-  mem_ptr += sizeof(int8_t) * (dataset->num_features_);
-
-  if (ArrayArgs<int8_t>::CheckAllZero(dataset->monotone_types_)) {
-    dataset->monotone_types_.clear();
-  }
-
-  if (!config_.feature_contri.empty()) {
-    CHECK(static_cast<size_t>(dataset->num_total_features_) == config_.feature_contri.size());
-    dataset->feature_penalty_.resize(dataset->num_features_);
-    for (int i = 0; i < dataset->num_total_features_; ++i) {
-      int inner_fidx = dataset->InnerFeatureIndex(i);
-      if (inner_fidx >= 0) {
-        dataset->feature_penalty_[inner_fidx] = config_.feature_contri[i];
-      }
-    }
-  } else {
-    const double* tmp_ptr_feature_penalty = reinterpret_cast<const double*>(mem_ptr);
-    dataset->feature_penalty_.clear();
-    for (int i = 0; i < dataset->num_features_; ++i) {
-      dataset->feature_penalty_.push_back(tmp_ptr_feature_penalty[i]);
-    }
-  }
-  mem_ptr += sizeof(double) * (dataset->num_features_);
-
-  if (ArrayArgs<double>::CheckAll(dataset->feature_penalty_, 1)) {
-    dataset->feature_penalty_.clear();
-  }
+  mem_ptr +=
+      VirtualFileWriter::AlignedSize(sizeof(int) * (dataset->num_groups_));
 
   if (!config_.max_bin_by_feature.empty()) {
-    CHECK(static_cast<size_t>(dataset->num_total_features_) == config_.max_bin_by_feature.size());
-    CHECK(*(std::min_element(config_.max_bin_by_feature.begin(), config_.max_bin_by_feature.end())) > 1);
+    CHECK_EQ(static_cast<size_t>(dataset->num_total_features_), config_.max_bin_by_feature.size());
+    CHECK_GT(*(std::min_element(config_.max_bin_by_feature.begin(), config_.max_bin_by_feature.end())), 1);
     dataset->max_bin_by_feature_.resize(dataset->num_total_features_);
     dataset->max_bin_by_feature_.assign(config_.max_bin_by_feature.begin(), config_.max_bin_by_feature.end());
   } else {
@@ -446,7 +424,8 @@ Dataset* DatasetLoader::LoadFromBinFile(const char* data_filename, const char* b
       dataset->max_bin_by_feature_.push_back(tmp_ptr_max_bin_by_feature[i]);
     }
   }
-  mem_ptr += sizeof(int32_t) * (dataset->num_total_features_);
+  mem_ptr += VirtualFileWriter::AlignedSize(sizeof(int32_t) *
+                                            (dataset->num_total_features_));
   if (ArrayArgs<int32_t>::CheckAll(dataset->max_bin_by_feature_, -1)) {
     dataset->max_bin_by_feature_.clear();
   }
@@ -456,22 +435,24 @@ Dataset* DatasetLoader::LoadFromBinFile(const char* data_filename, const char* b
   // write feature names
   for (int i = 0; i < dataset->num_total_features_; ++i) {
     int str_len = *(reinterpret_cast<const int*>(mem_ptr));
-    mem_ptr += sizeof(int);
+    mem_ptr += VirtualFileWriter::AlignedSize(sizeof(int));
     std::stringstream str_buf;
+    auto tmp_arr = reinterpret_cast<const char*>(mem_ptr);
     for (int j = 0; j < str_len; ++j) {
-      char tmp_char = *(reinterpret_cast<const char*>(mem_ptr));
-      mem_ptr += sizeof(char);
+      char tmp_char = tmp_arr[j];
       str_buf << tmp_char;
     }
+    mem_ptr += VirtualFileWriter::AlignedSize(sizeof(char) * str_len);
     dataset->feature_names_.emplace_back(str_buf.str());
   }
   // get forced_bin_bounds_
   dataset->forced_bin_bounds_ = std::vector<std::vector<double>>(dataset->num_total_features_, std::vector<double>());
   for (int i = 0; i < dataset->num_total_features_; ++i) {
     int num_bounds = *(reinterpret_cast<const int*>(mem_ptr));
-    mem_ptr += sizeof(int);
+    mem_ptr += VirtualFileWriter::AlignedSize(sizeof(int));
     dataset->forced_bin_bounds_[i] = std::vector<double>();
-    const double* tmp_ptr_forced_bounds = reinterpret_cast<const double*>(mem_ptr);
+    const double* tmp_ptr_forced_bounds =
+        reinterpret_cast<const double*>(mem_ptr);
     for (int j = 0; j < num_bounds; ++j) {
       double bound = tmp_ptr_forced_bounds[j];
       dataset->forced_bin_bounds_[i].push_back(bound);
@@ -570,9 +551,10 @@ Dataset* DatasetLoader::LoadFromBinFile(const char* data_filename, const char* b
 }
 
 
-Dataset* DatasetLoader::CostructFromSampleData(double** sample_values,
-                                               int** sample_indices, int num_col, const int* num_per_col,
-                                               size_t total_sample_size, data_size_t num_data) {
+Dataset* DatasetLoader::ConstructFromSampleData(double** sample_values,
+                                                int** sample_indices, int num_col, const int* num_per_col,
+                                                size_t total_sample_size, data_size_t num_data) {
+  CheckSampleSize(total_sample_size, static_cast<size_t>(num_data));
   int num_total_features = num_col;
   if (Network::num_machines() > 1) {
     num_total_features = Network::GlobalSyncUpByMax(num_total_features);
@@ -587,8 +569,8 @@ Dataset* DatasetLoader::CostructFromSampleData(double** sample_values,
     }
   }
   if (!config_.max_bin_by_feature.empty()) {
-    CHECK(static_cast<size_t>(num_col) == config_.max_bin_by_feature.size());
-    CHECK(*(std::min_element(config_.max_bin_by_feature.begin(), config_.max_bin_by_feature.end())) > 1);
+    CHECK_EQ(static_cast<size_t>(num_col), config_.max_bin_by_feature.size());
+    CHECK_GT(*(std::min_element(config_.max_bin_by_feature.begin(), config_.max_bin_by_feature.end())), 1);
   }
 
   // get forced split
@@ -618,13 +600,13 @@ Dataset* DatasetLoader::CostructFromSampleData(double** sample_values,
       bin_mappers[i].reset(new BinMapper());
       if (config_.max_bin_by_feature.empty()) {
         bin_mappers[i]->FindBin(sample_values[i], num_per_col[i], total_sample_size,
-                                config_.max_bin, config_.min_data_in_bin, filter_cnt,
+                                config_.max_bin, config_.min_data_in_bin, filter_cnt, config_.feature_pre_filter,
                                 bin_type, config_.use_missing, config_.zero_as_missing,
                                 forced_bin_bounds[i]);
       } else {
         bin_mappers[i]->FindBin(sample_values[i], num_per_col[i], total_sample_size,
                                 config_.max_bin_by_feature[i], config_.min_data_in_bin,
-                                filter_cnt, bin_type, config_.use_missing,
+                                filter_cnt, config_.feature_pre_filter, bin_type, config_.use_missing,
                                 config_.zero_as_missing, forced_bin_bounds[i]);
       }
       OMP_LOOP_EX_END();
@@ -666,16 +648,17 @@ Dataset* DatasetLoader::CostructFromSampleData(double** sample_values,
       if (config_.max_bin_by_feature.empty()) {
         bin_mappers[i]->FindBin(sample_values[start[rank] + i], num_per_col[start[rank] + i],
                                 total_sample_size, config_.max_bin, config_.min_data_in_bin,
-                                filter_cnt, bin_type, config_.use_missing, config_.zero_as_missing,
+                                filter_cnt, config_.feature_pre_filter, bin_type, config_.use_missing, config_.zero_as_missing,
                                 forced_bin_bounds[i]);
       } else {
         bin_mappers[i]->FindBin(sample_values[start[rank] + i], num_per_col[start[rank] + i],
                                 total_sample_size, config_.max_bin_by_feature[start[rank] + i],
-                                config_.min_data_in_bin, filter_cnt, bin_type, config_.use_missing,
+                                config_.min_data_in_bin, filter_cnt, config_.feature_pre_filter, bin_type, config_.use_missing,
                                 config_.zero_as_missing, forced_bin_bounds[i]);
       }
       OMP_LOOP_EX_END();
     }
+    OMP_THROW_EX();
     comm_size_t self_buf_size = 0;
     for (int i = 0; i < len[rank]; ++i) {
       if (ignore_features_.count(start[rank] + i) > 0) {
@@ -716,7 +699,7 @@ Dataset* DatasetLoader::CostructFromSampleData(double** sample_values,
     }
   }
   auto dataset = std::unique_ptr<Dataset>(new Dataset(num_data));
-  dataset->Construct(&bin_mappers, num_total_features, forced_bin_bounds, sample_indices, num_per_col, num_col, total_sample_size, config_);
+  dataset->Construct(&bin_mappers, num_total_features, forced_bin_bounds, sample_indices, sample_values, num_per_col, num_col, total_sample_size, config_);
   dataset->set_feature_names(feature_names_);
   return dataset.release();
 }
@@ -758,7 +741,7 @@ void DatasetLoader::CheckDataset(const Dataset* dataset) {
 std::vector<std::string> DatasetLoader::LoadTextDataToMemory(const char* filename, const Metadata& metadata,
                                                              int rank, int num_machines, int* num_global_data,
                                                              std::vector<data_size_t>* used_data_indices) {
-  TextReader<data_size_t> text_reader(filename, config_.header);
+  TextReader<data_size_t> text_reader(filename, config_.header, config_.file_load_progress_interval_bytes);
   used_data_indices->clear();
   if (num_machines == 1 || config_.pre_partition) {
     // read all lines
@@ -821,7 +804,7 @@ std::vector<std::string> DatasetLoader::SampleTextDataFromFile(const char* filen
                                                                int rank, int num_machines, int* num_global_data,
                                                                std::vector<data_size_t>* used_data_indices) {
   const data_size_t sample_cnt = static_cast<data_size_t>(config_.bin_construct_sample_cnt);
-  TextReader<data_size_t> text_reader(filename, config_.header);
+  TextReader<data_size_t> text_reader(filename, config_.header, config_.file_load_progress_interval_bytes);
   std::vector<std::string> out_data;
   if (num_machines == 1 || config_.pre_partition) {
     *num_global_data = static_cast<data_size_t>(text_reader.SampleFromFile(&random_, sample_cnt, &out_data));
@@ -894,12 +877,12 @@ void DatasetLoader::ConstructBinMappersFromTextData(int rank, int num_machines,
     dataset->num_total_features_ = Network::GlobalSyncUpByMax(dataset->num_total_features_);
   }
   if (!feature_names_.empty()) {
-    CHECK(dataset->num_total_features_ == static_cast<int>(feature_names_.size()));
+    CHECK_EQ(dataset->num_total_features_, static_cast<int>(feature_names_.size()));
   }
 
   if (!config_.max_bin_by_feature.empty()) {
-    CHECK(static_cast<size_t>(dataset->num_total_features_) == config_.max_bin_by_feature.size());
-    CHECK(*(std::min_element(config_.max_bin_by_feature.begin(), config_.max_bin_by_feature.end())) > 1);
+    CHECK_EQ(static_cast<size_t>(dataset->num_total_features_), config_.max_bin_by_feature.size());
+    CHECK_GT(*(std::min_element(config_.max_bin_by_feature.begin(), config_.max_bin_by_feature.end())), 1);
   }
 
   // get forced split
@@ -944,12 +927,12 @@ void DatasetLoader::ConstructBinMappersFromTextData(int rank, int num_machines,
       if (config_.max_bin_by_feature.empty()) {
         bin_mappers[i]->FindBin(sample_values[i].data(), static_cast<int>(sample_values[i].size()),
                                 sample_data.size(), config_.max_bin, config_.min_data_in_bin,
-                                filter_cnt, bin_type, config_.use_missing, config_.zero_as_missing,
+                                filter_cnt, config_.feature_pre_filter, bin_type, config_.use_missing, config_.zero_as_missing,
                                 forced_bin_bounds[i]);
       } else {
         bin_mappers[i]->FindBin(sample_values[i].data(), static_cast<int>(sample_values[i].size()),
                                 sample_data.size(), config_.max_bin_by_feature[i],
-                                config_.min_data_in_bin, filter_cnt, bin_type, config_.use_missing,
+                                config_.min_data_in_bin, filter_cnt, config_.feature_pre_filter, bin_type, config_.use_missing,
                                 config_.zero_as_missing, forced_bin_bounds[i]);
       }
       OMP_LOOP_EX_END();
@@ -988,13 +971,13 @@ void DatasetLoader::ConstructBinMappersFromTextData(int rank, int num_machines,
         bin_mappers[i]->FindBin(sample_values[start[rank] + i].data(),
                                 static_cast<int>(sample_values[start[rank] + i].size()),
                                 sample_data.size(), config_.max_bin, config_.min_data_in_bin,
-                                filter_cnt, bin_type, config_.use_missing, config_.zero_as_missing,
+                                filter_cnt, config_.feature_pre_filter, bin_type, config_.use_missing, config_.zero_as_missing,
                                 forced_bin_bounds[i]);
       } else {
         bin_mappers[i]->FindBin(sample_values[start[rank] + i].data(),
                                 static_cast<int>(sample_values[start[rank] + i].size()),
                                 sample_data.size(), config_.max_bin_by_feature[i],
-                                config_.min_data_in_bin, filter_cnt, bin_type,
+                                config_.min_data_in_bin, filter_cnt, config_.feature_pre_filter, bin_type,
                                 config_.use_missing, config_.zero_as_missing, forced_bin_bounds[i]);
       }
       OMP_LOOP_EX_END();
@@ -1039,8 +1022,8 @@ void DatasetLoader::ConstructBinMappersFromTextData(int rank, int num_machines,
       cp_ptr += bin_mappers[i]->SizesInByte();
     }
   }
-  sample_values.clear();
   dataset->Construct(&bin_mappers, dataset->num_total_features_, forced_bin_bounds, Common::Vector2Ptr<int>(&sample_indices).data(),
+                     Common::Vector2Ptr<double>(&sample_values).data(),
                      Common::VectorSize<int>(sample_indices).data(), static_cast<int>(sample_indices.size()), sample_data.size(), config_);
 }
 
@@ -1065,11 +1048,13 @@ void DatasetLoader::ExtractFeaturesFromMemory(std::vector<std::string>* text_dat
       ref_text_data[i].clear();
       // shrink_to_fit will be very slow in linux, and seems not free memory, disable for now
       // text_reader_->Lines()[i].shrink_to_fit();
+      std::vector<bool> is_feature_added(dataset->num_features_, false);
       // push data
       for (auto& inner_data : oneline_features) {
         if (inner_data.first >= dataset->num_total_features_) { continue; }
         int feature_idx = dataset->used_feature_map_[inner_data.first];
         if (feature_idx >= 0) {
+          is_feature_added[feature_idx] = true;
           // if is used feature
           int group = dataset->feature2group_[feature_idx];
           int sub_feature = dataset->feature2subfeature_[feature_idx];
@@ -1082,6 +1067,7 @@ void DatasetLoader::ExtractFeaturesFromMemory(std::vector<std::string>* text_dat
           }
         }
       }
+      dataset->FinishOneRow(tid, i, is_feature_added);
       OMP_LOOP_EX_END();
     }
     OMP_THROW_EX();
@@ -1109,10 +1095,12 @@ void DatasetLoader::ExtractFeaturesFromMemory(std::vector<std::string>* text_dat
       // shrink_to_fit will be very slow in linux, and seems not free memory, disable for now
       // text_reader_->Lines()[i].shrink_to_fit();
       // push data
+      std::vector<bool> is_feature_added(dataset->num_features_, false);
       for (auto& inner_data : oneline_features) {
         if (inner_data.first >= dataset->num_total_features_) { continue; }
         int feature_idx = dataset->used_feature_map_[inner_data.first];
         if (feature_idx >= 0) {
+          is_feature_added[feature_idx] = true;
           // if is used feature
           int group = dataset->feature2group_[feature_idx];
           int sub_feature = dataset->feature2subfeature_[feature_idx];
@@ -1125,6 +1113,7 @@ void DatasetLoader::ExtractFeaturesFromMemory(std::vector<std::string>* text_dat
           }
         }
       }
+      dataset->FinishOneRow(tid, i, is_feature_added);
       OMP_LOOP_EX_END();
     }
     OMP_THROW_EX();
@@ -1166,11 +1155,13 @@ void DatasetLoader::ExtractFeaturesFromFile(const char* filename, const Parser*
       }
       // set label
       dataset->metadata_.SetLabelAt(start_idx + i, static_cast<label_t>(tmp_label));
+      std::vector<bool> is_feature_added(dataset->num_features_, false);
       // push data
       for (auto& inner_data : oneline_features) {
         if (inner_data.first >= dataset->num_total_features_) { continue; }
         int feature_idx = dataset->used_feature_map_[inner_data.first];
         if (feature_idx >= 0) {
+          is_feature_added[feature_idx] = true;
           // if is used feature
           int group = dataset->feature2group_[feature_idx];
           int sub_feature = dataset->feature2subfeature_[feature_idx];
@@ -1183,11 +1174,12 @@ void DatasetLoader::ExtractFeaturesFromFile(const char* filename, const Parser*
           }
         }
       }
+      dataset->FinishOneRow(tid, i, is_feature_added);
       OMP_LOOP_EX_END();
     }
     OMP_THROW_EX();
   };
-  TextReader<data_size_t> text_reader(filename, config_.header);
+  TextReader<data_size_t> text_reader(filename, config_.header, config_.file_load_progress_interval_bytes);
   if (!used_data_indices.empty()) {
     // only need part of data
     text_reader.ReadPartAndProcessParallel(used_data_indices, process_fun);
@@ -1244,12 +1236,12 @@ std::vector<std::vector<double>> DatasetLoader::GetForcedBins(std::string forced
       std::stringstream buffer;
       buffer << forced_bins_stream.rdbuf();
       std::string err;
-      Json forced_bins_json = Json::parse(buffer.str(), err);
+      Json forced_bins_json = Json::parse(buffer.str(), &err);
       CHECK(forced_bins_json.is_array());
       std::vector<Json> forced_bins_arr = forced_bins_json.array_items();
       for (size_t i = 0; i < forced_bins_arr.size(); ++i) {
         int feature_num = forced_bins_arr[i]["feature"].int_value();
-        CHECK(feature_num < num_total_features);
+        CHECK_LT(feature_num, num_total_features);
         if (categorical_features.count(feature_num)) {
           Log::Warning("Feature %d is categorical. Will ignore forced bins for this  feature.", feature_num);
         } else {
