@@ -9,7 +9,7 @@ from io import BytesIO
 import numpy as np
 
 from .basic import Booster
-from .compat import (MATPLOTLIB_INSTALLED, GRAPHVIZ_INSTALLED, LGBMDeprecationWarning,
+from .compat import (MATPLOTLIB_INSTALLED, GRAPHVIZ_INSTALLED,
                      range_, zip_, string_type)
 from .sklearn import LGBMModel
 
@@ -329,7 +329,7 @@ def plot_metric(booster, metric=None, dataset_names=None,
     num_metric = len(metrics_for_one)
     if metric is None:
         if num_metric > 1:
-            msg = """more than one metric available, picking one to plot."""
+            msg = "More than one metric available, picking one to plot."
             warnings.warn(msg, stacklevel=2)
         metric, results = metrics_for_one.popitem()
     else:
@@ -374,7 +374,8 @@ def plot_metric(booster, metric=None, dataset_names=None,
     return ax
 
 
-def _to_graphviz(tree_info, show_info, feature_names, precision=3, constraints=None, **kwargs):
+def _to_graphviz(tree_info, show_info, feature_names, precision=3,
+                 orientation='horizontal', constraints=None, **kwargs):
     """Convert specified tree to graphviz instance.
 
     See:
@@ -441,7 +442,8 @@ def _to_graphviz(tree_info, show_info, feature_names, precision=3, constraints=N
             graph.edge(parent, name, decision)
 
     graph = Digraph(**kwargs)
-    graph.attr("graph", nodesep="0.05", ranksep="0.3", rankdir="LR")
+    rankdir = "LR" if orientation == "horizontal" else "TB"
+    graph.attr("graph", nodesep="0.05", ranksep="0.3", rankdir=rankdir)
     if "internal_count" in tree_info['tree_structure']:
         add(tree_info['tree_structure'], tree_info['tree_structure']["internal_count"])
     else:
@@ -469,9 +471,7 @@ def _to_graphviz(tree_info, show_info, feature_names, precision=3, constraints=N
 
 
 def create_tree_digraph(booster, tree_index=0, show_info=None, precision=3,
-                        old_name=None, old_comment=None, old_filename=None, old_directory=None,
-                        old_format=None, old_engine=None, old_encoding=None, old_graph_attr=None,
-                        old_node_attr=None, old_edge_attr=None, old_body=None, old_strict=False, **kwargs):
+                        orientation='horizontal', **kwargs):
     """Create a digraph representation of specified tree.
 
     .. note::
@@ -492,6 +492,9 @@ def create_tree_digraph(booster, tree_index=0, show_info=None, precision=3,
         'leaf_count', 'leaf_weight', 'data_percentage'.
     precision : int or None, optional (default=3)
         Used to restrict the display of floating point values to a certain precision.
+    orientation : string, optional (default='horizontal')
+        Orientation of the tree.
+        Can be 'horizontal' or 'vertical'.
     **kwargs
         Other parameters passed to ``Digraph`` constructor.
         Check https://graphviz.readthedocs.io/en/stable/api.html#digraph for the full list of supported parameters.
@@ -506,23 +509,6 @@ def create_tree_digraph(booster, tree_index=0, show_info=None, precision=3,
     elif not isinstance(booster, Booster):
         raise TypeError('booster must be Booster or LGBMModel.')
 
-    for param_name in ['old_name', 'old_comment', 'old_filename', 'old_directory',
-                       'old_format', 'old_engine', 'old_encoding', 'old_graph_attr',
-                       'old_node_attr', 'old_edge_attr', 'old_body']:
-        param = locals().get(param_name)
-        if param is not None:
-            warnings.warn('{0} parameter is deprecated and will be removed in 2.4 version.\n'
-                          'Please use **kwargs to pass {1} parameter.'.format(param_name, param_name[4:]),
-                          LGBMDeprecationWarning)
-            if param_name[4:] not in kwargs:
-                kwargs[param_name[4:]] = param
-    if locals().get('strict'):
-        warnings.warn('old_strict parameter is deprecated and will be removed in 2.4 version.\n'
-                      'Please use **kwargs to pass strict parameter.',
-                      LGBMDeprecationWarning)
-        if 'strict' not in kwargs:
-            kwargs['strict'] = True
-
     model = booster.dump_model()
     tree_infos = model['tree_info']
     if 'feature_names' in model:
@@ -540,14 +526,14 @@ def create_tree_digraph(booster, tree_index=0, show_info=None, precision=3,
     if show_info is None:
         show_info = []
 
-    graph = _to_graphviz(tree_info, show_info, feature_names, precision, monotone_constraints, **kwargs)
+    graph = _to_graphviz(tree_info, show_info, feature_names, precision,
+                         orientation, monotone_constraints, **kwargs)
 
     return graph
 
 
 def plot_tree(booster, ax=None, tree_index=0, figsize=None, dpi=None,
-              old_graph_attr=None, old_node_attr=None, old_edge_attr=None,
-              show_info=None, precision=3, **kwargs):
+              show_info=None, precision=3, orientation='horizontal', **kwargs):
     """Plot specified tree.
 
     .. note::
@@ -575,6 +561,9 @@ def plot_tree(booster, ax=None, tree_index=0, figsize=None, dpi=None,
         'leaf_count', 'leaf_weight', 'data_percentage'.
     precision : int or None, optional (default=3)
         Used to restrict the display of floating point values to a certain precision.
+    orientation : string, optional (default='horizontal')
+        Orientation of the tree.
+        Can be 'horizontal' or 'vertical'.
     **kwargs
         Other parameters passed to ``Digraph`` constructor.
         Check https://graphviz.readthedocs.io/en/stable/api.html#digraph for the full list of supported parameters.
@@ -590,22 +579,14 @@ def plot_tree(booster, ax=None, tree_index=0, figsize=None, dpi=None,
     else:
         raise ImportError('You must install matplotlib to plot tree.')
 
-    for param_name in ['old_graph_attr', 'old_node_attr', 'old_edge_attr']:
-        param = locals().get(param_name)
-        if param is not None:
-            warnings.warn('{0} parameter is deprecated and will be removed in 2.4 version.\n'
-                          'Please use **kwargs to pass {1} parameter.'.format(param_name, param_name[4:]),
-                          LGBMDeprecationWarning)
-            if param_name[4:] not in kwargs:
-                kwargs[param_name[4:]] = param
-
     if ax is None:
         if figsize is not None:
             _check_not_tuple_of_2_elements(figsize, 'figsize')
         _, ax = plt.subplots(1, 1, figsize=figsize, dpi=dpi)
 
     graph = create_tree_digraph(booster=booster, tree_index=tree_index,
-                                show_info=show_info, precision=precision, **kwargs)
+                                show_info=show_info, precision=precision,
+                                orientation=orientation, **kwargs)
 
     s = BytesIO()
     s.write(graph.pipe(format='png'))
